/*
 * test_compass.c
 *
 *  Created on: 2013/12/22
 *      Author: d10130
 *
 * 電子コンパス(HMC6352)のテスト用プログラム
 * エラーハンドリングが雑な部分が結構あります
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <debug.h>
#include "ui.h"
#include "i2c.h"
#include "compass.h"


double volatile angle = 0.0;

// 計測と記録をひたすら続けるスレッド
void compass_thread(void){

	I2CBus* bus;
	Compass* compass;
	FILE* pipeInput;
	char busName[128];
	char input[128];
	int busCount,deviceCount;
	int address;
	int intervalTime;
	int i;
	double angle_init,angle_def;

	/*初期設定*/
	intervalTime = 60e3;

	/* I2Cバスの検索 */
	busCount = 0;
	pipeInput = popen("ls /dev/ttyUSB-MIRS* 2> /dev/null","r");
	while( fgets(busName,sizeof(busName),pipeInput)!=NULL ){
		busName[strlen(busName)-1] = '\0';
		busCount++;
	}
	if(busCount == 0){
		printf("ERROR\n");
		fprintf(stderr,"ERROR: No I2C bus");
		exit(1);
	}else{
		printf("\n");
	}

	/* バスのオープン */
	if(busCount > 1){
		printf("I2C bus file");
		printf("/dev/ttyUSB-MIRS");
		if( fgets(input,sizeof(input),stdin) == NULL){
			printf("ERROR\n");
			exit(1);
		}
		if(input[strlen(input)-1] == '\n'){
			input[strlen(input)-1] = '\0';
		}
		sprintf(busName,"/dev/ttyUSB-MIRS%s",input);
	}
	bus = i2cBusOpen(busName);
	if(bus == NULL){
		printf("ERROR\n");
		exit(1);
	}

	/* I2Cデバイスの検索 */
	address = 0;
	deviceCount = 0;
	for(i=0; i<=I2C_ADDRESS_MAX; i++){
		if( i2cBusHasDevice(bus,i) ){
			address = i;
			deviceCount++;
		}
	}
	if(deviceCount == 0){
		printf("ERROR\n");
		fprintf(stderr,"ERROR: No Device\n");
		i2cBusClose(bus);
		exit(1);
	}

	/* 電子コンパスオープン */
	if(deviceCount > 1){
		printf("0x");
		if( fgets(input,sizeof(input),stdin) == NULL){
			printf("ERROR\n");
			exit(1);
		}
		address = strtol(input,NULL,16);
	}
	compass = compassOpen(bus,address);
	if(compass == NULL){
		printf("ERROR\n");
		i2cBusClose(bus);
		exit(1);
	}

	/*キャリブレーション*/
	printf("キャリブレーションスタート\n");
	compassStartCalibration(compass);
	while(getchar() != '\n');
	compassStopCalibration(compass);


	angle_init = compassGetAngle(compass);
	while(1){
		/* センサーモニター */
		printf("\n\n");
		fflush(stdout);
		while(1){
			if(compassGetAngle(compass) - angle_init < 0){
				angle_def = compassGetAngle(compass) - angle_init +360;
			}else{				
				angle_def = compassGetAngle(compass) - angle_init;
			}

			//補正(体育館)
			if(angle_def >= 270){
				angle = angle_def*((360-angle_def)/90);
			}else if(0 <= angle_def && angle_def < 180){
				angle = angle_def;
			}else{
				angle = angle_def + 10*((angle_def - 180)/90);
			}

			//

/*
			//補正(工作室)
			if(angle_def < 90){
				angle = angle_def + 2*(angle_def/90);
			}else if(angle_def < 180){
				angle = angle_def + 3*(angle_def/180);
			}else if(angle_def < 270){
				angle = angle_def + 10*(angle_def/270);
			}else{
				angle = angle_def + 10*((360-angle_def)/90);
			}
*/

			#ifdef __debug_compass__
			printf("angle = %6.2f [deg]\n",angle);
			#endif

			fflush(stdout);
			usleep(intervalTime);
		}
		printf("\n");
	}

	/* 電子コンパスのクローズ */
	compassClose(compass);

	/* バスクローズ */
	i2cBusClose(bus);

	exit(0);
}


