/*
 * test_i2c.c
 *
 *  Created on: 2013/12/22
 *      Author: d10130
 *
 * I2Cマスターボードのテスト用プログラム
 * エラーハンドリングが雑な部分が結構あります
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include "debug.h"
#include "i2c.h"
#include "ui.h"

/* 基本文字設定 */
const UIFont normalFont = {0,0,0,{UI_BLACK,UI_WHITE}};

/* 終了処理 */
void finalize(){
	UIFont font = normalFont;
	font.isBrink = 1;
	font.color.fg = UI_WHITE;
	font.color.bg = UI_RED;
	uiSetFont(font);

	uiCursorOff();
	uiMoveLines(uiGetMaxLines());
	uiPrintCenter("    Press any key to finish    ");
	fflush(stdout);
	while(uiGetKey() == EOF);

	uiReset();
	uiClear();
}

/* タイトル表示 */
void printTitle(){
	UIFont font = normalFont;

	font.isBrink = 1;
	font.color.fg = UI_WHITE;
	font.color.bg = UI_BLACK;
	uiSetFont(font);

	uiEraseLine(); printf("\n");
	uiEraseLine();
	uiPrintCenter("MIRS1301 I2C Master Board Test Program\n");
	uiEraseLine(); printf("\n");

	uiSetFont(normalFont);
}

int main(void){
	I2CBus* bus;
	I2CDevice* device;
	FILE* pipeInput;
	char busName[128];
	char input[128];
	uint8_t dataSize;
	uint8_t buf[256];
	int busCount;
	int deviceCount;
	int offsetCols;
	int address;
	int i,result;

	/* 初期設定 */
	atexit(finalize);
	signal(SIGINT,exit);
	uiClear();
	uiMove(0,0);
	uiSetFont(normalFont);
	uiErase();
	offsetCols = 20;

	/* 初期画面 */
	uiCursorOff();
	uiMove(uiGetMaxLines()/2,0);
	printTitle();
	uiCursorOff();
	printf("\n");
	uiPrintCenter("[ Press any key to start ]\n");
	while(uiGetKey() == EOF);
	uiCursorOn();
	uiErase();

	/* タイトル表示 */
	uiMove(0,0);
	printTitle();
	printf("\n");

	/* I2Cバスの検索 */
	uiSetFont(normalFont);
	printf("I2C bus list");
	uiMoveCols(offsetCols); printf(": ");
	busCount = 0;
	pipeInput = popen("ls /dev/ttyUSB-MIRS* 2> /dev/null","r");
	while( fgets(busName,sizeof(busName),pipeInput)!=NULL ){
		busName[strlen(busName)-1] = '\0';
		printf("%s ",busName);
		busCount++;
	}
	if(busCount == 0){
		printf("ERROR\n");
		ERROR_PRINT("No I2C bus");
		exit(1);
	}else{
		printf("\n");
	}

	/* バスのオープン */
	if(busCount > 1){
		printf("I2C bus file");
		uiMoveCols(offsetCols); printf("? ");
		printf("/dev/ttyUSB-MIRS");
		if( fgets(input,sizeof(input),stdin) == NULL){
			printf("ERROR\n");
			exit(1);
		}
		if(input[strlen(input)-1] == '\n'){
			input[strlen(input)-1] = '\0';
		}
		sprintf(busName,"/dev/ttyUSB-MIRS%s",input);
	}
	printf("I2C bus open");
	uiMoveCols(offsetCols); printf(": ");
	bus = i2cBusOpen(busName);
	if(bus == NULL){
		printf("ERROR\n");
		exit(1);
	}else{
		printf("OK\n");
	}

	/* I2Cデバイスの検索 */
	printf("I2C device list");
	uiMoveCols(offsetCols); printf(": ");
	address = 0;
	deviceCount = 0;
	for(i=0; i<=I2C_ADDRESS_MAX; i++){
		if( i2cBusHasDevice(bus,i) ){
			address = i;
			printf("0x%02X ",address);
			deviceCount++;
		}
	}
	if(deviceCount == 0){
		printf("ERROR\n");
		ERROR_PRINT("ERROR: No Device\n");
		i2cBusClose(bus);
		exit(1);
	}else{
		printf("\n");
	}
	/* I2Cデバイスオープン */
	if(deviceCount > 1){
		printf("I2C device address");
		uiMoveCols(offsetCols); printf("? ");
		printf("0x");
		if( fgets(input,sizeof(input),stdin) == NULL){
			printf("ERROR\n");
			exit(1);
		}
		address = strtol(input,NULL,16);
	}
	printf("I2C device open");
	uiMoveCols(offsetCols); printf(": ");
	device = i2cDeviceOpen(bus,address);
	if(device == NULL){
		printf("ERROR\n");
		i2cBusClose(bus);
		exit(1);
	}else{
		printf("OK\n");
	}

	/* 通信開始 */
	while(1){
		uiMove(0,0);
		uiErase();
		printTitle();
		printf("\n");
		printf("I2C bus");
		uiMoveCols(offsetCols); printf(": %s\n",busName);
		printf("I2C device");
		uiMoveCols(offsetCols); printf(": 0x%02X\n",address);
		printf("\n");
		printf("Read / Write [r/w]");
		uiMoveCols(offsetCols); printf("? ");
		if( fgets(input,sizeof(input),stdin) == NULL){ exit(1); }
		switch(input[0]){
			case 'r':
				printf("Data size");
				uiMoveCols(offsetCols); printf("? 0x");
				if( fgets(input,sizeof(input),stdin) == NULL){ exit(1); }
				dataSize = strtol(input,NULL,16);
				result = i2cRead(device,buf,dataSize);
				for(i=0; i<result; i++){
					printf("Read data [0x%02X]",i);
					uiMoveCols(offsetCols);
					printf(": 0x%02X\n",buf[i]);
				}
				if(result == dataSize){
					printf("Read");
					uiMoveCols(offsetCols); printf(": OK\n");
				}else{
					printf("Read");
					uiMoveCols(offsetCols); printf(": ERROR\n");
				}
				break;
			case 'w':
				printf("Data size");
				uiMoveCols(offsetCols); printf("? 0x");
				if( fgets(input,sizeof(input),stdin) == NULL){ exit(1); }
				dataSize = strtol(input,NULL,16);
				for(i=0; i<dataSize; i++){
					printf("Write data [0x%02X]",i);
					uiMoveCols(offsetCols); printf("? 0x");
					if( fgets(input,sizeof(input),stdin) == NULL){ exit(1); }
					buf[i] = strtol(input,NULL,16);
				}
				result = i2cWrite(device,buf,dataSize);
				if(result == dataSize){
					printf("Write");
					uiMoveCols(offsetCols); printf(": OK\n");
				}else{
					printf("Write");
					uiMoveCols(offsetCols); printf(": ERROR\n");
				}
				break;
			default:
				ERROR_PRINT("Invalid input\n");
				break;
		}
		printf("\n");
		printf("Countniue [y/n]");
		uiMoveCols(offsetCols); printf("? ");
		if( fgets(input,sizeof(input),stdin) == NULL){ exit(1); }
		if(input[0] != 'y'){
			break;
		}
	}
	printf("\n");

	/* I2Cデバイスクローズ */
	printf("I2C device close");
	uiMoveCols(offsetCols); printf(": ");
	i2cDeviceClose(device);
	printf("OK\n");

	/* バスクローズ */
	printf("I2C bus close");
	uiMoveCols(offsetCols); printf(": ");
	i2cBusClose(bus);
	printf("OK\n");

	return 0;
}
