/************************************************
*	色による領域の抽出を行う関数		*
*		src -> 入力画像(ch3, 8U)	*
*		dst -> 出力画像(ch1[bin], 8U)	*
*		th1_min〜th3_min -> 閾値の最小値	*
*		th1_max〜th3_max -> 閾値の最大値	*
*		*code -> 閾値の種類の指定	*
*			 NULL---RGBに変換して処理*
*			"HSV"---HSVに変換して処理*
************************************************/
#include <cxcore.h>
#include <cv.h>
#include "bmp.h"
#include <stdio.h>
#include "img.h"


double col_threshold(IplImage *src, IplImage *dst,
			 int th1_min, int th1_max,
			 int th2_min, int th2_max,
			 int th3_min, int th3_max,
			 char *code,  double *white_rate, int *lsl, int *lsr, int *rsl, int *rsr)
{

	int i, j;
	int nx, ny;
	unsigned char color1, color2, color3;
	int mode;
	int high_count=0;
	IplImage *colorImage;
	nx = src->width; ny = src->height;
	
	colorImage = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 3);
//RGBかHSVかで場合わけ
	if(!strcmp(code, "HSV")) mode = CV_BGR2HSV;
	else mode = CV_BGR2RGB;
	cvCvtColor(src, colorImage, mode);
//閾値処理
	for(i = 0; i < ny; i++){
		for(j = 0; j < nx; j++){
			color1 = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 0];
			color2 = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 1];
			color3 = (unsigned char)colorImage->imageData[3*nx*i + 3*j + 2];
			if(		(th1_min <= color1 && color1 <= th1_max)		//指定色調範囲内に入っているかの判断
				&&	(th2_min <= color2 && color2 <= th2_max)
				&&	(th3_min <= color3 && color3 <= th3_max))
			{
				dst->imageData[nx*i + j] = HIGH;				//そのピクセルが閾値の値の範囲内である
				high_count++;


				//検出されたピクセルがどの向きにあるのかを4種類の向きで場合わけ

				if(j <= nx/4){
							(*lsl)++;
				}else if(j <= 2*nx/4){
							(*lsr)++;				
				}else if(j <= 3*nx/4){
							(*rsl)++;
				}else{
							(*rsr)++;
				}

			}else{
					dst->imageData[nx*i + j] = LOW;
			}
		}
	}
	
	
	*white_rate = ((double)high_count/(nx*ny))*100;

#ifdef __debug_img__
	printf("high_count = %d \n",high_count);
#endif


//メモリ開放
	cvReleaseImage(&colorImage);
	return 0;
}
