/************************************************
*	塗りつぶしを行う関数(objFill)		*
*		src -> 入力画像(ch3, 8U)	*
*		dst -> 出力画像(ch1[bin], 8U/16S)*
*		x, y -> 塗りつぶし開始点		*
*		paint_color -> 塗りつぶす色	*
************************************************/
#include <cxcore.h>

void scan_line(	IplImage *src, int lx, int rx, int y, int col,
				CvPoint **End_ldx, CvPoint *bufstart, CvPoint *bufend){
	int nx=src->width;
	while(lx <= rx){
		for(; lx < rx; lx++)
			if((unsigned char)src->imageData[nx*y + lx] == col) break;
		if((unsigned char)src->imageData[nx*y + lx] != col) break;
		for(; lx <= rx; lx++)
			if((unsigned char)src->imageData[nx*y + lx] != col) break;
		(*End_ldx)->x = lx - 1;
		(*End_ldx)->y = y;
		(*End_ldx)++;
		if(*End_ldx == bufend) *End_ldx = bufstart;
	}
	return;
}

int fill(IplImage *src, IplImage *dst,
			int x, int y, int paint_color){
	int i;
	int nx=src->width, ny=src->height;
	int bufmax = 2*nx;
	int lx, rx, uy, dy;
	int col;
	int check_col;
	CvPoint Buffer[bufmax];
	CvPoint *Start_ldx, *End_ldx;
	
	if(	src->width != dst->width ||
		src->height != dst->height ||
		src->nChannels != 1 ||
		dst->nChannels != src->nChannels ||
		src->depth != IPL_DEPTH_8U){
			return(-1);
	}
	Start_ldx = Buffer;
	End_ldx = Buffer + 1;
	Start_ldx->x = x;
	Start_ldx->y = y;
	col = (unsigned char)src->imageData[nx*y + x];
	do{
		lx = rx = Start_ldx->x;
		uy = dy = Start_ldx->y;
		Start_ldx++;
		if(Start_ldx == &Buffer[bufmax]) Start_ldx = Buffer;
		if(dst->depth == IPL_DEPTH_8U){
			check_col = (unsigned char)dst->imageData[nx*uy + lx];
		}else{
			check_col = ((short*)(dst->imageData))[nx*uy + lx];
		}
		if(check_col == paint_color) continue;
		while(rx < nx){
			if((unsigned char)src->imageData[nx*uy + (rx+1)] != col) break;
			rx++;
		}
		while(lx > 0){
			if((unsigned char)src->imageData[nx*uy + (lx-1)] != col) break;
			lx--;
		}
		for(i = lx; i <= rx; i++){
			if(dst->depth == IPL_DEPTH_8U){
				dst->imageData[nx*uy + i] = (unsigned char)paint_color;
			}else if((unsigned int)dst->depth == IPL_DEPTH_16S){
				((short*)(dst->imageData))[nx*uy + i] = paint_color;
			}
		}
		uy--;
		if(uy >= 0){
			scan_line(src, lx, rx, uy, col, &End_ldx, Buffer, &Buffer[bufmax]);
		}
		dy++;
		if(dy <= ny){
			scan_line(src, lx, rx, dy, col, &End_ldx, Buffer, &Buffer[bufmax]);
		}
	}while(Start_ldx != End_ldx);
	return 0;
}
