#ifndef __MOTOR_H__
#define __MOTOR_H__

#include <stdint.h>

// 標準のKp
#define MOTOR_DEFAULT_KP 50
// 標準のKi
#define MOTOR_DEFAULT_KI 3
// 標準の制御周期
#define MOTOR_DEFAULT_PERIOD 100 // [ms]


// モーター型
struct motor_t;
typedef struct motor_t Motor;

// モーターとの通信を開始してモーターポインタを初期化する。
// 戻り値 成功:0 失敗:-1
extern int motor_open(const char* dev_file,Motor** p_motor);
// モーターとの通信を終了する。
extern void motor_close(Motor* motor);
// ロータリーエンコーダの値[pulse]の初期化と取得
extern void motor_set_count(Motor* motor, int32_t count_init);
extern int32_t motor_get_count(Motor* motor);
// モーターの回転速度[pulse/s]の変更と取得
// -128~127[pulse/制御周期]を超える値を設定しようとした場合は切り捨て
extern void motor_set_speed(Motor* motor, float pulse_par_sec);
extern float motor_get_speed(Motor* motor);
// ブレーキを掛ける
extern void motor_brake(Motor* motor);
// Kp,Kiの変更と取得
extern void motor_set_kp(Motor* motor,uint8_t kp);
extern void motor_set_ki(Motor* motor,uint8_t ki);
extern uint8_t motor_get_kp(Motor* motor);
extern uint8_t motor_get_ki(Motor* motor);



// [ 以下の5つの関数の戻り値 ]
// 成功すれば，0が返り，失敗すると負数が返る．
// 失敗したときには，errnoにエラーコードがセットされる．
// reqパケットの内容が不正な時にはEBADRQCがセットされる.
// その他の引数が不正な場合，EINVALがセットされる.
// その他のエラーコードについては，read(2)，write(2)を参照．

// モータに制御パラメータを送る
extern int motor_send_control_packet (int fd, uint8_t speed, uint8_t kp, uint8_t ki);
// 現在のモータの状態を得る
extern int motor_recv_status_packet (int fd, int8_t *speed, uint16_t *pwm, int32_t *round);
// モータに回転を開始させる
extern int motor_send_start_packet (int fd);
// モータの回転を停止させる
extern int motor_send_stop_packet (int fd);
// REカウント値のクリア
extern int motor_send_clear_packet(int fd);
// モータの制御周期を変更する 単位はms
extern int motor_send_period_packet (int fd, uint16_t period);


#endif
