#include "mirs.h"
#include "run.h"
#include "motor.h"

static int x_l,x_r;
static char v_l, v_r, v_lm, v_rm;
static int c_left,c_right;
static float speed_l, speed_r;
static int dir=1;
static MotorInfo m_info_l, m_info_r;
static int diff_prev=0, diff_sum=0;
static int speed;

char run_pid_v(int diff){
	float kp=0.3, kd=0.0, ki=0.0;
	char ret;
	diff_sum+=diff;
	ret= kp*diff + kd*(diff-diff_prev) + ki*diff_sum;
	diff_prev=diff;
	return ret;
}
/* 左右モータの速度補正データの初期化*/
void run_pid_v_init(){
	diff_prev=0;
	diff_sum=0;
}

int run_stop(){
	motor_send_stop_packet (fd1);
	motor_send_stop_packet (fd2);
	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
	return 0;
}

void run_straight_set(void *arg){

	int *cmd=(int*)arg;

	speed=cmd[0];

	v_l = speed; v_r = speed; 
	speed_l = speed; speed_r = speed; 

	
	#ifdef __DEBUG_RUN__
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){ v_l=VLMAX; v_r=VLMAX; }
		else{ v_l = -VLMAX; v_r = -VLMAX; }
	}else if(abs(speed_l) < VLMIN){
		if(speed_l > 0){ v_l=VLMIN; v_r=VLMIN; }
		else{ v_l = -VLMIN; v_r = -VLMIN; }
	}

	v_lm = v_l; v_rm = v_r;

	if(v_l<0){dir=-1;}

	run_pid_v_init();

	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);

}

void run_straight(void){

	char vmod;
	int diff;

	motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
	motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
	c_left=m_info_l.puls;
	c_right=m_info_r.puls;
	//printf("puls\tl = %d, r = %d\n", c_left, c_right);
	x_l=(int)(c_left*t_par);
	x_r=(int)(c_right*t_par);

	#ifdef __DEBUG_RUN__
		printf("x_l = %d, x_r = %d\n", x_l, x_r);
		printf("spped_l = %d\t speed_r = %d\t pwm_l =%d\t pwm_r =%d\n", m_info_l.speed,  m_info_l.speed,  m_info_l.pwm,  m_info_r.pwm );
	#endif


	diff= x_l - x_r;
	vmod = run_pid_v(diff);
	v_lm=v_l - vmod;
	v_rm=v_r + vmod;
	motor_send_control_packet(fd1,v_lm,Kp,Ki);
	motor_send_control_packet(fd2,v_rm,Kp,Ki);

}


void run_rotate_set(void *arg){

	int *cmd=(int*)arg;

	speed=cmd[0];

	v_l = speed; v_r = speed; 
	speed_l = speed; speed_r = speed; 

	#ifdef __DEBUG_RUN__
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){ v_l=VLMAX; v_r=VLMAX; }
		else{ v_l = -VLMAX; v_r = -VLMAX; }
	}else if(abs(speed_l) < VLMIN){
		if(speed_l > 0){ v_l=VLMIN; v_r=VLMIN; }
		else{ v_l = -VLMIN; v_r = -VLMIN; }
	}

	v_lm = v_l; v_rm = v_r;

	if(v_l<0){dir=-1;}

	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
	printf("send start\n");
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);


	v_l = -speed; v_r = speed; 
	speed_l = -speed; speed_r = speed; 
	#ifdef __DEBUG_RUN__
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){
			v_l = VLMAX; v_r = -VLMAX;
		}else{
			v_l = -VLMAX; v_r = VLMAX;
		}
	} else if(abs(speed_l)<VLMIN){
		if(speed_l > 0){
			v_l = VLMIN; v_r = -VLMIN;
		}else{
			v_l = -VLMIN; v_r = VLMIN;
		}
	}

	if(v_l>0){dir=-1;}

	run_pid_v_init();

	v_lm = v_l; v_rm = v_r;

	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);

}


void run_rotate(void){

	char vmod;
	int diff;

	motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
	motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));

	c_left=m_info_l.puls;
	c_right=m_info_r.puls;
	//printf("puls\tl = %d, r = %d\n", c_left, c_right);
	x_l=(int)(c_left*t_par);
	x_r=(int)(c_right*t_par);

	#ifdef __DEBUG_RUN__
		printf("x_l = %d, x_r = %d\n", x_l, x_r);
		printf("spped_l = %d\t speed_r = %d\t pwm_l =%d\t pwm_r =%d\n", m_info_l.speed,  m_info_l.speed,  m_info_l.pwm,  m_info_r.pwm );
	#endif

	diff= x_l + x_r;
	vmod = run_pid_v(diff);
	v_lm=v_l - vmod;
	v_rm=v_r - vmod;

	motor_send_control_packet(fd1,v_lm,Kp,Ki);
	motor_send_control_packet(fd2,v_rm,Kp,Ki);

}

static int dis_l=0,dis_r=0;
static int rot, radius,rot_dir;
static int rot_direction;
static float ratio_l, ratio_r;
static float MR=MIRS_RADIUS;
static int set_time;

void run_circle_set(void *arg){


	int *cmd=(int*)arg;

	rot=cmd[0];
	set_time=cmd[1];
	radius=cmd[2];
	rot_dir=cmd[3];

	if(rot_dir==0){
		dis_l=rot*PI/180*(radius+MIRS_RADIUS);
		dis_r=rot*PI/180*(radius-MIRS_RADIUS);
		rot_direction=1;
	}
	if(rot_dir==1){
		dis_l=rot*PI/180*(radius-MIRS_RADIUS);
		dis_r=rot*PI/180*(radius+MIRS_RADIUS);
		rot_direction=-1;
	}

	speed_l=100*dis_l/(t_par*set_time);
	speed_r=100*dis_r/(t_par*set_time);

	v_l = (int)speed_l;
	v_r = (int)speed_r;

	ratio_l= (radius + MR*rot_direction)/ radius;
	ratio_r= (radius - MR*rot_direction)/ radius;

	#ifdef __DEBUG_RUN__
		printf("circle distance : dis_l= %d[mm]  dis_r=%d[mm]\n",dis_l, dis_r);
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif


	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){
			v_l=VLMAX;
		}else{
			v_l=-VLMAX;
		}
	}
	if(abs(speed_r) > VLMAX){
		if(speed_r > 0){
			v_r=VLMAX;
		}else{
			v_r=-VLMAX;
		}
	}
	if(abs(speed_l) < VLMIN){
		if(speed_l > 0){
			v_r+=(VLMIN-v_l);
			v_l=VLMIN;
		}else{
			v_r-=(VLMIN+v_l);
			v_l=-VLMIN;
		}
	}
	if(abs(speed_r) < VLMIN){
		if(speed_r > 0){
			v_l+=(VLMIN-v_r);
			v_r=VLMIN;
		}else{
			v_l-=(VLMIN+v_r);
			v_r=-VLMIN;
		}
	}

	v_lm = v_l; v_rm = v_r;

	if(v_l<0){dir=-1;}

	run_pid_v_init();

	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);


}

void run_circle(void){

	int x_lr,dist;
	float theta_l, theta_r;
	char vmod;
	int diff;

	motor_send_control_packet(fd1,v_r,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);

	c_left=0; c_right=0;

	motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
	motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));

	//recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
	//recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
	//motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
	////printf("puls\tl = %ld, r = %ld\n", m_info_l.puls, m_info_r.puls);

	c_left=m_info_l.puls;
	c_right=m_info_r.puls;
	//printf("puls\tl = %d, r = %d\n", c_left, c_right);
	x_l=(int)(c_left*t_par);
	x_r=(int)(c_right*t_par);
	//printf("vl = %d, vr = %d, xl = %d, xr = %d\n", v_l, v_r, x_l, x_r);

	x_lr=(x_l+x_r)/2;
	dist=(dis_l+dis_r)/2;

	//printf("dis_err=%d\n", dis_err);
	theta_l=x_l/(radius+MR*rot_direction);
	theta_r=x_r/(radius-MR*rot_direction);
	#ifdef __DEBUG_RUN__
		printf("theta_l=%f \t  theta_r=%f\n",theta_l*180/3.14, theta_r*180/3.14);
		printf("spped_l = %d\t speed_r = %d\t pwm_l =%d\t pwm_r =%d\n", m_info_l.speed,  m_info_l.speed,  m_info_l.pwm,  m_info_r.pwm );
	#endif

	diff=(theta_l-theta_r)*radius;
	//printf("diff=%d\n",diff);
	vmod = run_pid_v(diff);
	v_lm=v_l - vmod*ratio_l;
	v_rm=v_r + vmod*ratio_r;

	motor_send_control_packet(fd1,v_lm,Kp,Ki);
	motor_send_control_packet(fd2,v_rm,Kp,Ki);

	//printf("v_lm = %d, v_rm = %d, xl = %d, xr = %d\n", v_lm, v_rm, x_l, x_r);


}
