/*
 * run_rotate.c
 *
 *  Created on: 2013/07/28
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include "mirs.h"
#include "motor.h"
#include "run.h"
#include "compass.h"

// 速度補正用ゲイン
#define V_KP 0.01
#define V_KI 0.00
#define V_KD 0.00
// 位置補正用ゲイン
#define X_KP 0.005
#define X_KI 0.001 //0.0006
#define X_KD 0.00

struct run_status_t {
	// 使用するモーター
	 Motor* motor_l;
	 Motor* motor_r;
	// 左右のエンコーダーの初期値
	 int32_t count_l_init;
	 int32_t count_r_init;
	// 速度補正用
	 int32_t v_err;
	 int32_t v_err_sum;
	 int32_t v_err_last;

	//angle
	 double angle_init;
	// 位置補正用
	 float x_err;
	 float x_err_sum;
	 float x_err_last;
};



// 使用するモーター,初期位置の設定
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
RunStatus* run_rotate_init(Motor* m_l,Motor* m_r){
	RunStatus* st;
	st = (RunStatus*)malloc(sizeof(RunStatus));
	if(st == NULL){
		return NULL;
	}

	st->motor_l = m_l;
	st->motor_r = m_r;

	st->count_l_init = motor_get_count(st->motor_l);
	st->count_r_init = motor_get_count(st->motor_r);

	st->angle_init = angle;

	st->v_err = 0;
	st->v_err_sum = 0;
	st->v_err_last = 0;

	st->x_err = 0;
	st->x_err_sum = 0;
	st->x_err_last = 0;

	return st;
}



// 繰り返し呼ぶと回転。目標角速度 angle_v[deg/s]。戻り値:回転角度[deg]
float run_rotate(float angle_v,RunStatus* st){
	int32_t count_l,count_r; // [pulse]
	float v,dv; // [pulse/s]

	// 半径と角速度から速度を計算
	v = angle_v * M_PI / 180.0 * MIRS_BODY_RADIUS / MIRS_DISTANCE_TO_PULSE_RATIO;

	count_l =  motor_get_count(st->motor_l) - st->count_l_init;
	count_r =  motor_get_count(st->motor_r) - st->count_r_init;

	st->v_err_last = st->v_err;
	st->v_err = count_r + count_l;
	st->v_err_sum += st->v_err;

	dv = 0.0;
	dv += V_KP * st->v_err;
	dv += V_KI * st->v_err_sum;
	dv += V_KD * (st->v_err - st->v_err_last);

	motor_set_speed(st->motor_l,-(v+dv) );
	motor_set_speed(st->motor_r,  v-dv );


	return angle;
	//return MIRS_DISTANCE_TO_PULSE_RATIO * (count_r - count_l)/2.0 / MIRS_BODY_RADIUS * 180.0 / M_PI;
}



// 繰り返し呼ぶと回転。目標回転角度 angle_ref[deg], 上限角速度 max_angle_speed[deg/s]。戻り値:回転角度[deg]
float run_rotate_to(float angle_ref,float max_angle_speed,RunStatus* st){
	float v;
	/*角度の範囲修正*/
	while(angle_ref >= 360.0){
		angle_ref -= 360.0;
	}
	while(angle_ref < 0.0){
		angle_ref += 360.0;
	}

	
	max_angle_speed = fabs(max_angle_speed);

	v = 0.0;
	v += X_KP * st->x_err;
	v += X_KI * st->x_err_sum;
	v += X_KD * (st->x_err - st->x_err_last);

	if(fabs(v) > max_angle_speed){
		v = (v > 0)?(max_angle_speed):(-max_angle_speed);
	}

	run_rotate(-v,st);

	st->x_err_last = st->x_err;
	if(angle_ref - angle > 180){		//0-360の壁を越えるかどうかで回転方向を修正
		st->x_err = angle_ref - angle - 360;	
	}else if(angle_ref - angle < -180){
		st->x_err = angle_ref - angle + 360;
	}else{
		st->x_err = angle_ref - angle;
	}
	st->x_err_sum += st->x_err;
	if(fabs(st->x_err) < fabs(st->x_err_last)-1){
		st->x_err_sum = 0;
	}

	return angle;
}



// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
void run_rotate_finish(RunStatus* st){
	motor_brake(st->motor_l);
	motor_brake(st->motor_r);
	free(st);
}
