#include "run.h"
#include "mirs.h"
#include "motor.h"

int theta_data;


static int c_left,c_right;
static int dis,set_time;
static float speed_l, speed_r;
static int x_l,x_r;
static int dir=1;
static char v_l, v_r, v_lm, v_rm;
static MotorInfo m_info_l, m_info_r;


static int MOTOR_SLEEP=120; // 制御ループの sleep 時間 [msec]
//static int DIS_FACT=0.2; //残り距離に対して速度を決める時の係数

static char mod_v2(int diff){
        float kp=0.2;
        char ret;
        ret= kp*diff ;
        return ret;
}


int run_sp(char sel,int *dt){

	motor_send_start_packet (fd1);
	motor_send_start_packet (fd2);
	if(sel == 's'){
		//printf("t start\n");
		run_sp_straight(dt);
	}else if(sel == 'r'){
		run_sp_rotate(dt);
	}else if(sel == 'c'){
		run_sp_circle(dt);
	}else{
		printf("command_errer\n");
	}
	
	motor_send_stop_packet (fd1);
	motor_send_stop_packet (fd2);
	return 0;
}


void run_sp_straight(void *arg){

	int *cmd=(int*)arg;
	dis=cmd[0];
	set_time = cmd[1];
	
	speed_l=100.*dis/(t_par*set_time);
	speed_r=100.*dis/(t_par*set_time);
	v_l = (int)speed_l;
	v_r = (int)speed_r;

	#ifdef __DEBUG_RUN__
		printf("dis = %d   spped_l= %5.2f  t_par=%5.2f  set_time=%d\n", dis,speed_l,t_par,set_time);
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){ v_l=VLMAX; v_r=VLMAX; }
		else{ v_l = -VLMAX; v_r = -VLMAX; }
	}else if(abs(speed_l) < VLMIN){
		if(speed_l > 0){ v_l=VLMIN; v_r=VLMIN; }
		else{ v_l = -VLMIN; v_r = -VLMIN; }
	}

	v_lm = v_l; v_rm = v_r;	

	if(v_l<0){dir=-1;}

	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);

	printf("send start\n");
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);
	usleep(MOTOR_SLEEP*1000);

	run_pid_v_init();

	//printf("end\n");
	c_left=0; c_right=0;
	while(1){
		//printf("recv start\n");
		motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
		motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
		//printf("puls\tl = %ld, r = %ld\n", m_info_l.puls, m_info_r.puls);
		//printf("end\n");
		c_left=m_info_l.puls;
		c_right=m_info_r.puls;
		//printf("puls\tl = %d, r = %d\n", c_left, c_right);
		x_l=(int)(c_left*t_par);
		x_r=(int)(c_right*t_par);
		//printf("vl = %d, vr = %d, xl = %d, xr = %d\n", v_l, v_r, x_l, x_r);

		char vmod, v_st;
		int diff, dis_err, dis_m;

		if(abs(x_l-dis) < 20 && abs(x_r-dis) < 20){
	#ifdef __DEBUG_RUN__
			printf("xl = %d, xr = %d\n", x_l, x_r);
	#endif
			break;
		}

		dis_m= (x_r + x_l)/2;
		dis_err = (dis - dis_m)*dir;
		//printf("dis_err=%d\n", dis_err);

		// 目標値との差(dis_err)がある距離以下になったら、その差に応じて減速する。
		if( dis_err >= 150) {
			diff= x_l - x_r;
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod;
			v_rm=v_r + vmod;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}else{
			v_st=mod_v2(dis_err)*dir;
			v_l=v_st; v_r=v_st;
			diff= x_l - x_r;
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod;
			v_rm=v_r + vmod;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}
	#ifdef __DEBUG_RUN__
		printf("v_lm = %d, v_rm = %d, xl = %d, xr = %d\n", v_lm, v_rm, x_l, x_r);
	#endif

		usleep(MOTOR_SLEEP*1000);
	}
	
	run_stop();
}

void run_sp_rotate(void *arg){

	int *cmd=(int*)arg;
	double angle = cmd[0]*PI/180;
	dis = (int)(angle*MIRS_RADIUS);
	set_time = cmd[1];

	
	//speed_l = (int)(-100*dis/(t_par*set_time));
	//speed_r = (int)(100*dis/(t_par*set_time));
	speed_l = -100*dis/(t_par*set_time);
	speed_r = 100*dis/(t_par*set_time);

	v_l = (char)speed_l;
	v_r = (char)speed_r;
	#ifdef __DEBUG_RUN__
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){ 
			v_l = VLMAX; v_r = -VLMAX;
		}else{ 
			v_l = -VLMAX; v_r = VLMAX;
		}
	} else if(abs(speed_l)<VLMIN){
		if(speed_l > 0){ 
			v_l = VLMIN; v_r = -VLMIN;
		}else{ 
			v_l = -VLMIN; v_r = VLMIN;
		}
	}

	if(v_l>0){dir=-1;}

	v_lm = v_l; v_rm = v_r;	

	//dis=abs(dis);


//	motor_send_start_packet (fd1);
//	motor_send_start_packet (fd2);
	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);
	usleep(MOTOR_SLEEP*1000);
	c_left=0; c_right=0;
	v_lm=v_l; v_rm=v_r;

	#ifdef __DEBUG_RUN__
		printf("rotation distance [mm] = %d\n",dis);
	#endif

	run_pid_v_init();

	c_left=0; c_right=0;
	while(1){
		//printf("recv start\n");
		motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
		motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
		//printf("puls\tl = %ld, r = %ld\n", m_info_l.puls, m_info_r.puls);
		//printf("end\n");		
		c_left=m_info_l.puls;
		c_right=m_info_r.puls;
		//printf("puls\tl = %d, r = %d\n", c_left, c_right);
		x_l=(int)(c_left*t_par);
		x_r=(int)(c_right*t_par);
		//printf("vl = %d, vr = %d, xl = %d, xr = %d\n", v_l, v_r, x_l, x_r);
		//if(v_l <= 0 || v_r <= 0) break;

		char vmod, v_st;
		int diff, dis_err, dis_m;

		//if(abs(x_l-dis) < 20 && abs(x_r-dis) < 20){

		dis_m= (x_r - x_l)/2;
		//dis_err = (dis - dis_m)*dir;
		dis_err = (dis - dis_m)/2.*180./MIRS_RADIUS*dir;
		//dis_err = dis - dis_m;
		//printf("dis_err=%d\n", dis_err);

		float theta;
		theta=(float)dis_m/MIRS_RADIUS*180./PI;
	#ifdef __DEBUG_RUN__
		printf("theta=%8.1f\n", theta);
	#endif
		theta_data=theta;

		if(abs(theta - cmd[0])< 3){  // 誤差指定3度
			//printf("xl = %d, xr = %d\n", x_l, x_r);
			break;
		}


		// 目標値との差(dis_err)がある角度以下になったら、その差に応じて減速する。
		if( dis_err >= 10) {  // in degree
			//diff= x_l - x_r;
			diff= x_l + x_r;
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod;
			v_rm=v_r - vmod;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}else{
			v_st=mod_v2(dis_err)*dir;
			//v_st=dis_err*DIS_FACT*dir;
			v_l=-v_st; v_r=v_st;
			//diff= x_l - x_r;
			diff= x_l + x_r;
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod;
			//v_rm=v_r + vmod;
			v_rm=v_r - vmod;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}
	#ifdef __DEBUG_RUN__
		printf("v_lm = %d, v_rm = %d, xl = %d, xr = %d\n", v_lm, v_rm, x_l, x_r);
	#endif

		usleep(MOTOR_SLEEP*1000);
	}


	usleep(130000);
	run_stop();
	mtask_flag=1;

}

void run_sp_circle(void *arg){

	int dis_l=0,dis_r=0;
	int rot, radius,rot_dir;
	int rot_direction;
	int x_lr,dist;
	float theta_l, theta_r;
	float ratio_l, ratio_r;
	float MR=MIRS_RADIUS;

	int *cmd=(int*)arg;

	rot=cmd[0];
	set_time=cmd[1];
	radius=cmd[2];
	rot_dir=cmd[3];


	if(rot_dir==0){
		dis_l=rot*PI/180*(radius+MIRS_RADIUS);
		dis_r=rot*PI/180*(radius-MIRS_RADIUS);
		rot_direction=1;
	}
	if(rot_dir==1){
		dis_l=rot*PI/180*(radius-MIRS_RADIUS);
		dis_r=rot*PI/180*(radius+MIRS_RADIUS);
		rot_direction=-1;
	}
	
	//speed_l=(int)(100*dis_l/(t_par*set_time));
	//speed_r=(int)(100*dis_r/(t_par*set_time));
	speed_l=100*dis_l/(t_par*set_time);
	speed_r=100*dis_r/(t_par*set_time);
	
	//v_l = (char)speed_l;
	//v_r = (char)speed_r;
	v_l = (int)speed_l;
	v_r = (int)speed_r;

	ratio_l= (radius + MR*rot_direction)/ radius;
	ratio_r= (radius - MR*rot_direction)/ radius;

	#ifdef __DEBUG_RUN__
		printf("circle distance : dis_l= %d[mm]  dis_r=%d[mm]\n",dis_l, dis_r);
		printf("original vl = %d, vr = %d\n", v_l, v_r);
	#endif

	if(abs(speed_l) > VLMAX){
		if(speed_l > 0){ 
			v_l=VLMAX;
		}else{
			v_l=-VLMAX;
		}
	}
	if(abs(speed_r) > VLMAX){
		if(speed_r > 0){ 
			v_r=VLMAX;
		}else{
			v_r=-VLMAX;
		}
	}
	if(abs(speed_l) < VLMIN){
		if(speed_l > 0){ 
			v_r+=(VLMIN-v_l);
			v_l=VLMIN; 
		}else{
			v_r-=(VLMIN+v_l);
			v_l=-VLMIN;
		}
	}
	if(abs(speed_r) < VLMIN){
		if(speed_r > 0){ 
			v_l+=(VLMIN-v_r);
			v_r=VLMIN; 
		}else{
			v_l-=(VLMIN+v_r);
			v_r=-VLMIN;
		}
	}

	v_lm = v_l; v_rm = v_r;	

	if(v_l<0){dir=-1;}


	motor_send_clear_packet(fd1);
	motor_send_clear_packet(fd2);
	//printf("send start\n");
	motor_send_control_packet(fd1,v_l,Kp,Ki);
	motor_send_control_packet(fd2,v_r,Kp,Ki);

	usleep(MOTOR_SLEEP*1000);

	run_pid_v_init();

	//printf("end\n");
	c_left=0; c_right=0;

	while(1){
		//pthread_testcancel();
		//printf("recv start\n");
		motor_recv_status_packet(fd1, &(m_info_l.speed), &(m_info_l.pwm), &(m_info_l.puls));
		motor_recv_status_packet(fd2, &(m_info_r.speed), &(m_info_r.pwm), &(m_info_r.puls));
		////printf("puls\tl = %ld, r = %ld\n", m_info_l.puls, m_info_r.puls);
		//printf("end\n");		
		c_left=m_info_l.puls;
		c_right=m_info_r.puls;
		//printf("puls\tl = %d, r = %d\n", c_left, c_right);
		x_l=(int)(c_left*t_par);
		x_r=(int)(c_right*t_par);
		//printf("vl = %d, vr = %d, xl = %d, xr = %d\n", v_l, v_r, x_l, x_r);
		//if(v_l <= 0 || v_r <= 0) break;

		x_lr=(x_l+x_r)/2;
		dist=(dis_l+dis_r)/2;

		char vmod, v_st;
		int diff, dis_err, dis_m;

		if(abs(x_l-dis_l) < 20 && abs(x_r-dis_r) < 20){
	#ifdef __DEBUG_RUN__
			printf("xl = %d, xr = %d\n", x_l, x_r);
	#endif
			 break;
		}

		dis_m= (x_r + x_l)/2;
		dis_err = (dist - dis_m)*dir;
		//printf("dis_err=%d\n", dis_err);
		theta_l=x_l/(radius+MR*rot_direction);
		theta_r=x_r/(radius-MR*rot_direction);
	#ifdef __DEBUG_RUN__
		printf("theta_l=%f \t theta_r=%f\n",theta_l*180/3.14, theta_r*180/3.14);
	#endif

		// 目標値との差(dis_err)がある距離以下になったら、その差に応じて減速する。
		if( dis_err >= 150) {
			//diff=(theta_l-theta_r)*300; 
			diff=(theta_l-theta_r)*radius; 
	#ifdef __DEBUG_RUN__
			printf("diff=%d\n",diff);
	#endif
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod*ratio_l;
			v_rm=v_r + vmod*ratio_r;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}else{
			v_st=mod_v2(dis_err)*dir;
			v_l=v_st*ratio_l; 
			v_r=v_st*ratio_r;
			diff=(theta_l-theta_r)*radius; 
			vmod = run_pid_v(diff);
			v_lm=v_l - vmod*ratio_l;
			v_rm=v_r + vmod*ratio_r;
			motor_send_control_packet(fd1,v_lm,Kp,Ki);
			motor_send_control_packet(fd2,v_rm,Kp,Ki);
		}
	#ifdef __DEBUG_RUN__
		printf("v_lm = %d, v_rm = %d, xl = %d, xr = %d\n", v_lm, v_rm, x_l, x_r);
	#endif

		usleep(MOTOR_SLEEP*1000);
	}
	run_stop();
}




