/*
 * run_trace_wall.c
 *
 *  Created on: 2013/07/29
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include "run.h"
#include "motor.h"
#include "mirs.h"

// ゲイン
#define KP 0.1
#define KI 0.0
#define KD 0.0

struct run_status_t {
	// 使用するモーター
	 Motor* motor_l;
	 Motor* motor_r;
	 // 使用するUSS
	 uint8_t uss_l;
	 uint8_t uss_r;
	// 左右のエンコーダーの初期値
	 int32_t count_l_init;
	 int32_t count_r_init;
	// 偏差
	 int32_t err;
	 int32_t err_sum;
	 int32_t err_last;
};



// 使用するモーター,USSの設定
// uss_get(uss_l) < uss_get(uss_r) のとき反時計方向に旋回
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
RunStatus* run_trace_wall_init(Motor* m_l,Motor* m_r,uint8_t uss_l,uint8_t uss_r){
	RunStatus* st;
	st = (RunStatus*)malloc(sizeof(RunStatus));
	if(st == NULL){
		return NULL;
	}

	st->motor_l = m_l;
	st->motor_r = m_r;

	st->count_l_init = motor_get_count(st->motor_l);
	st->count_r_init = motor_get_count(st->motor_r);

	st->uss_l = uss_l;
	st->uss_r = uss_r;

	st->err = 0;
	st->err_sum = 0;
	st->err_last = 0;

	return st;
}




// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
void run_trace_wall_finish(RunStatus* st){
	motor_brake(st->motor_l);
	motor_brake(st->motor_r);
	free(st);
}
