/*
 * ui.c
 *
 *  Created on: 2013/12/23
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>
#include <sys/time.h>
#include "ui.h"

/* 標準入力から非カノニカル、エコー無し、待ち時間無しで1文字取得
 * 戻値: 取得した文字 (取得できなかった場合はEOF)           */
int uiGetKey(){
	int ch;
	int result;
	struct termios newt,oldt;
	tcgetattr(0,&oldt);
	newt = oldt;
	newt.c_lflag &= ~(ICANON|ECHO);
	newt.c_cc[VTIME] = 0;
	newt.c_cc[VMIN] = 0;
	tcsetattr(0,TCSANOW,&newt);
	result = read(0,&ch,1);
	tcsetattr(0,TCSAFLUSH,&oldt);
	if(result == 0){
		ch = EOF;
	}
	return ch;
}

/* 画面サイズの取得 */
int uiGetMaxLines(){
	static struct timeval time0 = {0,0};
	static int max = -1;
	struct timeval time1,time;
	FILE* process;
	char str[10];

	gettimeofday(&time1,NULL);
	timersub(&time1,&time0,&time);
	if(time.tv_sec > 0){
		process = popen("tput lines","r");
		if(process == NULL){
			return -1;
		}
		if(fgets(str,sizeof(str),process) == NULL){
			max = -1;
		}else{
			max = atoi(str) - 1;
		}
		pclose(process);
		gettimeofday(&time0,NULL);
	}
	return max;
}
int uiGetMaxCols(){
	static struct timeval time0 = {0,0};
	static int max = -1;
	struct timeval time1,time;
	FILE* process;
	char str[10];

	gettimeofday(&time1,NULL);
	timersub(&time1,&time0,&time);
	if(time.tv_sec > 0){
		process = popen("tput cols","r");
		if(process == NULL){
			return -1;
		}
		if(fgets(str,sizeof(str),process) == NULL){
			max = -1;
		}else{
			max = atoi(str) - 1;
		}
		pclose(process);
		gettimeofday(&time0,NULL);
	}
	return max;
}
/* エコーバックの有無 */
void uiEchoOn(){
	struct termios newt,oldt;
	tcgetattr(0,&oldt);
	newt = oldt;
	newt.c_lflag |= (ECHO);
	tcsetattr(0,TCSANOW,&newt);
}
void uiEchoOff(){
	struct termios newt,oldt;
	tcgetattr(0,&oldt);
	newt = oldt;
	newt.c_lflag &= ~(ECHO);
	tcsetattr(0,TCSANOW,&newt);
}
/* カーソルの表示/非表示 */
void uiCursorOn(){
	printf("\x1b[?25h");
}
void uiCursorOff(){
	printf("\x1b[?25l");
}

/* カーソルの移動(絶対座標) */
void uiMove(int lines,int cols){
	if(lines < 0){ lines = 0; }
	if(cols < 0){ cols = 0; }
	printf("\x1b[%d;%dH",lines,cols);
}
void uiMoveLines(int lines){
	int max;
	max = uiGetMaxLines();
	if(max > 0){
		uiMoveRelative(-max,0);
		uiMoveRelative(lines,0);
	}
}
void uiMoveCols(int cols){
	int max;
	max = uiGetMaxCols();
	if(max > 0){
		uiMoveRelative(0,-max);
		uiMoveRelative(0,cols);
	}
}
/* カーソルの移動(相対座標) */
void uiMoveRelative(int lines,int cols){
	if(lines > 0){
		printf("\x1b[%dB",lines);
	}else if(lines < 0){
		printf("\x1b[%dA",-lines);
	}
	if(cols > 0){
		printf("\x1b[%dC",cols);
	}else if(cols < 0){
		printf("\x1b[%dD",-cols);
	}
}

/* system("clear")と同等 */
void uiClear(){
	printf("\x1b[0;0H\x1b[2J");
}
/* 画面全体の削除 (カーソル位置不動) */
void uiErase(){
	printf("\x1b[0J\x1b[1J");
}
/* カーソルのある行全体を削除 (カーソル位置不動) */
void uiEraseLine(){
	printf("\x1b[0K\x1b[1K");
}
/* カーソルより先を削除 (カーソル位置不動) */
void uiEraseToBottom(){
	printf("\x1b[0J");
}
/* カーソルより前を削除 (カーソル位置不動) */
void uiEraseFromTop(){
	printf("\x1b[1J");
}
/* カーソルから行末(End Of Line)まで削除 (カーソル位置不動) */
void uiEraseToEOL(){
	printf("\x1b[0K");
}
/* 行の開始点からカーソルまで削除 (カーソル位置不動) */
void uiEraseFromSOL(){
	printf("\x1b[1K");
}

/* 文字をデフォルトの状態に戻す */
void uiReset(){
	struct termios oldt;
	tcgetattr(0,&oldt);
	oldt.c_lflag |= (ICANON|ECHO);
	tcsetattr(0,TCSAFLUSH,&oldt);
	printf("\x1b[?25h\x1b[0m");
}
/* 文字の設定 */
void uiSetFont(UIFont font){
	printf("\x1b[0m");
	if(font.isBrink){
		printf("\x1b[1m");
	}
	if(font.isUnderLined){
		printf("\x1b[4m");
	}
	if(font.isReverse){
		printf("\x1b[7m");
	}
	printf("\x1b[3%d;4%dm",font.color.fg,font.color.bg);
}

/* 右揃えで出力する */
void uiPrintRight(char* str){
	int max;
	max = uiGetMaxCols();
	uiMoveCols(max - strlen(str));
	printf("%s",str);
}
/* 中央揃えで出力する */
void uiPrintCenter(char* str){
	int max;
	max = uiGetMaxCols();
	uiMoveCols((max - strlen(str))/2);
	printf("%s",str);
}
/* 左揃えで出力する */
void uiPrintLeft(char* str){
	uiMoveCols(0);
	printf("%s",str);
}
