/*
 * ui.h
 *
 *  Created on: 2013/12/23
 *      Author: d10130
 *
 * curses.h使うのが面倒なとき用。
 * エスケープシーケンスベタ書きなので端末によっては動かない。
 * 256色パレットはCUIモードで使えないので未対応。
 */

#ifndef UI_H_
#define UI_H_

/* 色リスト */
typedef enum {
	UI_BLACK,
	UI_RED,
	UI_GREEN,
	UI_YELLOW,
	UI_BLUE,
	UI_MAGENTA,
	UI_CYAN,
	UI_WHITE
} UIColor;

/* 文字情報構造体 */
typedef struct {
	int isBrink;      /* ハイライト または 太字 */
	int isUnderLined; /* 下線 */
	int isReverse;    /* 反転 */
	struct {
		UIColor fg; /* 文字色 */
		UIColor bg; /* 背景色 */
	} color;
} UIFont;

/* 標準入力から非カノニカル、エコー無し、待ち時間無しで1文字取得
 * 戻値: 取得した文字                                       */
int uiGetKey();

/* 画面サイズの取得 エラー時は-1を返す*/
int uiGetMaxLines();
int uiGetMaxCols();

/* エコーバックの有無 */
void uiEchoOn();
void uiEchoOff();
/* カーソルの表示/非表示 */
void uiCursorOn();
void uiCursorOff();

/* カーソルの移動(絶対座標) */
void uiMove(int lines,int cols);
void uiMoveLines(int lines);
void uiMoveCols(int cols);
/* カーソルの移動(相対座標) */
void uiMoveRelative(int lines,int cols);

/* system("clear")と同等 */
void uiClear();
/* 画面全体の削除 (カーソル位置不動) */
void uiErase();
/* カーソルのある行全体を削除 (カーソル位置不動) */
void uiEraseLine();
/* カーソルより先を削除 (カーソル位置不動) */
void uiEraseToBottom();
/* カーソルより前を削除 (カーソル位置不動) */
void uiEraseFromTop();
/* カーソルから行末(End Of Line)まで削除 (カーソル位置不動) */
void uiEraseToEOL();
/* 行の開始点からカーソルまで削除 (カーソル位置不動) */
void uiEraseFromSOL();

/* デフォルトの状態に戻す */
void uiReset();
/* 文字の設定 */
void uiSetFont(UIFont font);

/* 右揃えで出力する */
void uiPrintRight(char* str);
/* 中央揃えで出力する */
void uiPrintCenter(char* str);
/* 左揃えで出力する */
void uiPrintLeft(char* str);

#endif /* UI_H_ */
