/*
	uss_read:
		Read distance data mesuared by USS.
*/


#include "mirs.h"
#include "uss.h"

int fd0;
volatile int uss[4];
static int uss_get(int);


pthread_t uss_task;
pthread_attr_t attr_uss;

int uss_read(int uss_no){
	return uss_get(uss_no);
}

void uss_read_all(int max_num){
	int data;
	int i;

	#ifdef __DEBUG_USS__
		printf("max_num=%d\n",max_num);
		printf("uss all start  \n");
	#endif
	
	for(i=0;i<max_num;i++){
		data = uss_get(i+1);
		if(data == -1){
			printf("uss_read: uss_no error\n");
		}
		usleep(50*1000); //30msec以上の wait を入れないと、隣り合うUSSが干渉する。
		uss[i] = data;
	#ifdef __DEBUG_USS__
		printf("uss[%d] = %d\n", i+1, uss[i]);
	#endif
	}
	#ifdef __DEBUG_USS__
	printf("uss12_diff = %d\n", uss[0]-uss[1]);
	#endif
	
	return;
}

void uss_read_thread(int max_num){
	int data;
	int i;
	#ifdef __DEBUG_USS__
		printf("max_num=%d\n",max_num);
		printf("uss all start  \n");
	#endif
	
	while(1){
		for(i=0;i<max_num;i++){
			pthread_testcancel();
			data = uss_get(i+1);
			if(data == -1){
				printf("uss_read: uss_no error\n");
				pthread_exit((void*)-1);
			}
			usleep(70*1000); //30msec以上の wait を入れないと、隣り合うUSSが干渉する。
			uss[i] = data;
	#ifdef __DEBUG_USS__
			printf("uss[%d] = %d\n", i+1, uss[i]);
	#endif
		}
	#ifdef __DEBUG_USS__
		printf("uss12_diff = %d\n", uss[0]-uss[1]);
	#endif
	}
	
	pthread_exit(0);
	return;
}

int uss_get(int uss_num){
	unsigned char rs_rbuffer[2];
	unsigned char rs_sbuffer[2];
	int c=0,x;
	int data = 9999;
	int i=0;
	//int rval;
	struct termios oldtio;
	if(uss_num < 0 || uss_num > 4) return(-1);
	sprintf((char*)rs_sbuffer,"%c",(char)uss_num);
//	sprintf(rs_sbuffer,"%c",(char)uss_num);
	// 毎回 uss デバイをを open/close する必要がある 
	fd0=open(S_PORT,O_RDWR | O_NOCTTY | O_NONBLOCK);

	set_serial_port(fd0,&oldtio);
	put_serial_string(fd0,(char*)rs_sbuffer);

	// 2バイトのデータ受けとるまで無限ループする。
	while(1){
		c=get_serial_char(fd0);
		rs_rbuffer[i] = c;
		i++;
		if(i==2){
			//2バイト目のデータが 0xff であったら応答なしとして、dataに9999を入れる
			if(rs_rbuffer[1]==0xff){
				//printf("\"%s\" receive no response\n",S_PORT);
				data = 9999;
				break;
			}
			c=rs_rbuffer[1];
			x=c<<8;
			c=rs_rbuffer[0];
			data=x+c+20;
			break;
		}
	}

	close_serial_port(fd0,&oldtio);

	return (data);
}
